"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Packager = exports.sort = exports.deserialize = exports.delay = void 0;
const global_ids_1 = require("./global-ids");
const gql_1 = require("./gql");
const query_1 = require("./gql/query");
const minilinks_1 = require("./minilinks");
const delay = (time) => new Promise(res => setTimeout(() => res(null), time));
exports.delay = delay;
function deserialize(pckg, errors = []) {
    var _a, _b, _c, _d, _e, _f, _g;
    const data = pckg.data.map(l => (Object.assign(Object.assign({}, l), { value: l.value ? Object.assign({}, l.value) : undefined })));
    const containsHash = {};
    let counter = 1;
    for (let l = 0; l < data.length; l++) {
        const item = data[l];
        if (item.type) {
            containsHash[item.id] = item;
            item.id = counter++;
        }
    }
    for (let l = 0; l < data.length; l++) {
        const item = data[l];
        if (!item.type && item.id)
            item.id = (_a = containsHash[item.id]) === null || _a === void 0 ? void 0 : _a.id;
        if (item.type)
            item.type = (_b = containsHash[item.type]) === null || _b === void 0 ? void 0 : _b.id;
        if (item.from)
            item.from = ((_c = containsHash[item.from]) === null || _c === void 0 ? void 0 : _c.id) || 0;
        if (item.to)
            item.to = ((_d = containsHash[item.to]) === null || _d === void 0 ? void 0 : _d.id) || 0;
    }
    const containsArray = Object.keys(containsHash);
    for (let c = 0; c < containsArray.length; c++) {
        const contain = containsArray[c];
        data.push({
            id: counter++,
            type: (_e = containsHash === null || containsHash === void 0 ? void 0 : containsHash['Contain']) === null || _e === void 0 ? void 0 : _e.id,
            from: (_f = containsHash === null || containsHash === void 0 ? void 0 : containsHash['package']) === null || _f === void 0 ? void 0 : _f.id,
            to: (_g = containsHash[contain]) === null || _g === void 0 ? void 0 : _g.id,
            value: { value: contain },
        });
    }
    return { data, errors };
}
exports.deserialize = deserialize;
function sort(pckg, data, errors = []) {
    let sorted = [];
    if (pckg.strict) {
        sorted = data;
    }
    else {
        for (let i = 0; i < data.length; i++) {
            const item = data[i];
            if (item === null || item === void 0 ? void 0 : item.type) {
                const first = sorted.findIndex((l, index) => l.from == item.id || l.to == item.id || l.type == item.id);
                const max = sorted.reduce((p, l, index) => l.id == item.type || l.id == item.from || l.id === item.to ? index : p, 0);
                sorted.splice((!!~first && first < max ? first : max) + 1, 0, item);
            }
            else {
                const max = sorted.reduce((p, l, index) => l.id == item.id ? index : p, 0);
                sorted.splice((max) + 1, 0, item);
            }
        }
    }
    return { sorted };
}
exports.sort = sort;
class Packager {
    constructor(client) {
        this.client = client;
        global.packager = this;
    }
    fetchTypeToTablesHash(types) {
        var _a, _b;
        return __awaiter(this, void 0, void 0, function* () {
            const q = yield this.client.query((0, query_1.generateQuery)({
                queries: [
                    (0, query_1.generateQueryData)({ tableName: 'links', returning: `id values: out(where: { type_id: { _eq: ${global_ids_1.GLOBAL_ID_TABLE_VALUE} } }) { id: to_id }`, variables: { where: {
                                type_id: { _eq: global_ids_1.GLOBAL_ID_TABLE },
                                out: { type_id: { _eq: global_ids_1.GLOBAL_ID_TABLE_VALUE }, to_id: { _in: types } },
                            } } }),
                ],
                name: 'LOAD_TYPES_TABLES_HASH',
            }));
            const tables = (_b = (_a = (q)) === null || _a === void 0 ? void 0 : _a.data) === null || _b === void 0 ? void 0 : _b.q0;
            const result = {};
            for (let t = 0; t < tables.length; t++) {
                const table = tables[t];
                for (let i = 0; i < table.values.length; i++) {
                    const type = table.values[i].id;
                    if (types.includes(type))
                        result[type] = table.id;
                }
            }
            console.log(types, result);
            return result;
        });
    }
    insertItem(items, item, errors, mutated) {
        var _a, _b, _c, _d;
        return __awaiter(this, void 0, void 0, function* () {
            yield (0, exports.delay)(2000);
            if (item.type) {
                const linkInsert = yield this.client.mutate((0, gql_1.generateSerial)({
                    actions: [(0, gql_1.insertMutation)('links', { objects: { type_id: item.type, from_id: item.from || 0, to_id: item.to || 0 } })],
                    name: 'IMPORT_PACKAGE_LINK',
                }));
                if (linkInsert === null || linkInsert === void 0 ? void 0 : linkInsert.errors)
                    errors.push(linkInsert === null || linkInsert === void 0 ? void 0 : linkInsert.errors);
                mutated[item.id] = true;
                const id = (_d = (_c = (_b = (_a = linkInsert === null || linkInsert === void 0 ? void 0 : linkInsert.data) === null || _a === void 0 ? void 0 : _a.m0) === null || _b === void 0 ? void 0 : _b.returning) === null || _c === void 0 ? void 0 : _c[0]) === null || _d === void 0 ? void 0 : _d.id;
                items.filter(i => i.id === item.id).forEach(l => l.id = id);
                items.filter(i => i.from === item.id).forEach(l => l.from = id);
                items.filter(i => i.to === item.id).forEach(l => l.to = id);
                items.filter(i => i.type === item.id).forEach(l => l.type = id);
                item.id = id;
            }
            if (item.value) {
                yield (0, exports.delay)(1000);
                const valueLink = items.find(i => i.id === item.id && !!i.type);
                if (!valueLink)
                    errors.push(`Link ${item.id} for value not founded.`);
                else {
                    const tables = yield this.fetchTypeToTablesHash([+valueLink.type]);
                    const valueInsert = yield this.client.mutate((0, gql_1.generateSerial)({
                        actions: [(0, gql_1.insertMutation)(`table${tables[valueLink.type]}`, { objects: Object.assign({ link_id: valueLink.id }, item.value) })],
                        name: 'IMPORT_PACKAGE_VALUE',
                    }));
                    if (valueInsert === null || valueInsert === void 0 ? void 0 : valueInsert.errors)
                        errors.push(valueInsert === null || valueInsert === void 0 ? void 0 : valueInsert.errors);
                }
            }
        });
    }
    insertItems(pckg, data, errors = [], mutated = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            for (let i = 0; i < data.length; i++) {
                const item = data[i];
                yield this.insertItem(data, item, errors, mutated);
                if (errors.length)
                    return;
            }
        });
    }
    import(pckg) {
        return __awaiter(this, void 0, void 0, function* () {
            const errors = [];
            const { data } = deserialize(pckg, errors);
            if (errors.length)
                return { errors };
            const { sorted } = sort(pckg, data, errors);
            if (errors.length)
                return { errors };
            yield this.insertItems(pckg, sorted, errors);
            if (errors.length)
                return { errors };
            return { errors };
        });
    }
    selectLinks(options) {
        var _a, _b;
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield this.client.query((0, query_1.generateQuery)({
                queries: [
                    (0, query_1.generateQueryData)({ tableName: 'links', returning: `
          id type_id from_id to_id value
          type {
            id value
            contains: in(where: { type: { value: { _contains: { value: ${global_ids_1.GLOBAL_NAME_CONTAIN} } } }, from: { type: { value: { _contains: { value: ${global_ids_1.GLOBAL_NAME_PACKAGE} } } } } }) {
              id
              package: from {
                id value
              }
            }
          }
          from {
            id
            contains: in(where: { type: { value: { _contains: { value: ${global_ids_1.GLOBAL_NAME_CONTAIN} } } }, from: { type: { value: { _contains: { value: ${global_ids_1.GLOBAL_NAME_PACKAGE} } } } } }) {
              id
              package: from {
                id value
              }
            }
          }
          to {
            id
            contains: in(where: { type: { value: { _contains: { value: ${global_ids_1.GLOBAL_NAME_CONTAIN} } } }, from: { type: { value: { _contains: { value: ${global_ids_1.GLOBAL_NAME_PACKAGE} } } } } }) {
              id
              package: from {
                id value
              }
            }
          }
        `, variables: { where: {
                                _or: [
                                    { id: { _eq: options.packageLinkId } },
                                    { type: { value: { _contains: { value: global_ids_1.GLOBAL_NAME_CONTAIN } } }, from: { id: { _eq: options.packageLinkId } } },
                                    { in: { type: { value: { _contains: { value: global_ids_1.GLOBAL_NAME_CONTAIN } } }, from: { id: { _eq: options.packageLinkId } } } },
                                ]
                            } } }),
                ],
                name: 'LOAD_PACKAGE_LINKS',
            }));
            return (0, minilinks_1.minilinks)((_b = (_a = (result)) === null || _a === void 0 ? void 0 : _a.data) === null || _b === void 0 ? void 0 : _b.q0);
        });
    }
    serialize(globalLinks, options, pckg) {
        var _a, _b, _c, _d;
        return __awaiter(this, void 0, void 0, function* () {
            let counter = 1;
            const containsByTo = {};
            const links = [];
            for (let l = 0; l < globalLinks.links.length; l++) {
                const link = globalLinks.links[l];
                if (((_b = (_a = link.type) === null || _a === void 0 ? void 0 : _a.value) === null || _b === void 0 ? void 0 : _b.value) === global_ids_1.GLOBAL_NAME_CONTAIN) {
                    containsByTo[+link.to_id] = link;
                }
                else
                    links.push(link);
            }
            for (let l = 0; l < links.length; l++) {
                const link = links[l];
                link._id = link.id;
                link.id = ((_d = (_c = containsByTo[+link.id]) === null || _c === void 0 ? void 0 : _c.value) === null || _d === void 0 ? void 0 : _d.value) ? containsByTo[+link.id].value.value : counter++;
                for (let li = 0; li < link.out.length; li++) {
                    const _link = link.out[li];
                    _link.from_id = link.id;
                }
                for (let li = 0; li < link.in.length; li++) {
                    const _link = link.in[li];
                    _link.to_id = link.id;
                }
                for (let li = 0; li < link.typed.length; li++) {
                    const _link = link.typed[li];
                    _link.type_id = link.id;
                }
                let value;
                if (link === null || link === void 0 ? void 0 : link.value) {
                    const _e = link.value, { id: __id, link_id: __link_id } = _e, _value = __rest(_e, ["id", "link_id"]);
                    value = _value;
                }
                const newLink = {
                    id: link.id,
                    type: link.type_id,
                };
                if (link.from)
                    newLink.from = link.from.id;
                if (link.to)
                    newLink.to = link.to.id;
                if (value)
                    newLink.value = value;
                pckg.data.push(newLink);
            }
        });
    }
    export(options) {
        var _a, _b;
        return __awaiter(this, void 0, void 0, function* () {
            const globalLinks = yield this.selectLinks(options);
            const packageLink = (_a = globalLinks.links) === null || _a === void 0 ? void 0 : _a.find(l => { var _a, _b; return ((_b = (_a = l === null || l === void 0 ? void 0 : l.type) === null || _a === void 0 ? void 0 : _a.value) === null || _b === void 0 ? void 0 : _b.value) === global_ids_1.GLOBAL_NAME_PACKAGE; });
            const packageName = (_b = packageLink === null || packageLink === void 0 ? void 0 : packageLink.value) === null || _b === void 0 ? void 0 : _b.value;
            const pckg = {
                package: packageName,
                data: [],
                strict: false,
                errors: [],
            };
            yield this.serialize(globalLinks, options, pckg);
            return pckg;
        });
    }
}
exports.Packager = Packager;
//# sourceMappingURL=packager.js.map